#define _USE_MATH_DEFINES
 
#include "wSolutionInterp.h"
#include <iostream>
#include <unordered_set>
#include <fstream>
#include <iomanip>

using namespace dart;

SolutionInterp::SolutionInterp(unsigned int _meshFactor)
{
  meshFactor = _meshFactor;
  CoarseMeshInit = false;
  
} // end SolutionInterp


SolutionInterp::~SolutionInterp()
{   
  std::cout << "~SolutionInterp()\n";
} // end ~SolutionInterp

void SolutionInterp::InputCoarseMesh(std::vector<double> _CoarseLocMesh, std::vector<double> _CoarseGlobMesh)
{
  CoarseLocMesh = _CoarseLocMesh;
  CoarseGlobMesh = _CoarseGlobMesh;
  CoarseMeshInit = true;
}
void SolutionInterp::InputFineMesh(std::vector<double> _fineLocMesh, std::vector<double> _fineGlobMesh)
{
  FineLocMesh=_fineLocMesh;
  FineGlobMesh=_fineGlobMesh;
}

std::vector<double> SolutionInterp::InterpolateToFineMesh(std::vector<double> cVector, size_t fineSize)
{
  if (meshFactor<=1)
  {
    std::cout << "Can not create finer mesh while meshFactor is " << meshFactor << std::endl;
    return cVector;
  }
  else
  {
    std::vector<double> fVector(fineSize-1);
    double dv;
    for (size_t cPoint=0; cPoint < cVector.size()-1; ++cPoint)
    {
      dv = cVector[cPoint + 1] - cVector[cPoint];
      for (size_t fPoint=0; fPoint<meshFactor; ++fPoint)
      {
        fVector[meshFactor * cPoint + fPoint] = cVector[cPoint] + fPoint * dv / meshFactor;
      }
    }
    fVector.push_back(cVector.back());
    return fVector; 
  }
}

std::vector<double> SolutionInterp::BlowingFineToCoarse(std::vector<double> fVector, size_t coarseSize)
{
  std::vector<double> cVector(coarseSize);

  for (size_t cCell=0; cCell<cVector.size(); ++cCell)
  {
    for (size_t k=0; k<meshFactor; ++k)
    {
      cVector[cCell] += fVector[cCell*meshFactor+k];
    }
    cVector[cCell]/=meshFactor;
  }  
  cVector.back() = fVector.back();
  return cVector;
}

std::vector<double> SolutionInterp::VarsFineToCoarse(std::vector<double> fVector, size_t coarseSize)
{
  std::vector<double> cVector(coarseSize);

  for (size_t cPoint=0; cPoint<cVector.size(); ++cPoint)
  {
    cVector[cPoint] = fVector[cPoint * meshFactor];
  }
  cVector.back() = fVector.back();
  return cVector;
}


std::vector<double> SolutionInterp::CreateFineMesh(std::vector<double> cMesh)
{
  if (meshFactor<1)
  {
    throw std::runtime_error("SolutionInterp::CreateFineMesh: Mesh factor should be greater than 1.");
  }

  else
  {

    double dx;
    
    /* Local markers */

    std::vector<double> fMesh;
    for (size_t iPoint=0; iPoint<cMesh.size()-1; ++iPoint)
    {
      dx = cMesh[iPoint + 1] - cMesh[iPoint];

      for(size_t iRefinedPoint = 0; iRefinedPoint < meshFactor; ++iRefinedPoint)
      {
        fMesh.push_back(cMesh[iPoint] + iRefinedPoint * dx/meshFactor);
      }
    }
    fMesh.push_back(cMesh.back());
    return fMesh;
  }
}

void SolutionInterp::ResetmeshFactor(unsigned int newmeshFactor)
{
  meshFactor = newmeshFactor;
}