################################################################################
#                                                                              #
#                            DARTFlo viscous module file                       #
#                      Validation: Class that displays the results             #
#                          and compare them with given results                 #
#                                                                              #
# Author: Paul Dechamps                                                        #
# Université de Liège                                                          #
# Date: 2021.09.10                                                             #
#                                                                              #
################################################################################


# ------------------------------------------------------------------------------
#  Imports
# ------------------------------------------------------------------------------
import numpy as np
import matplotlib.pyplot as plt

class Validation:
    def __init__(self,_xfoilFileName = None):
        self.xfoilFileName=_xfoilFileName
    def main(self, dartcl, wData, wakeblVectors = None, xWake = None):
        # Load xfoil data

        xFoilIn = 0
        cl_xfoil = 0
        cd_xfoil = 0

        if self.xfoilFileName is not None:

            xFoilIn = 1
            with open('/Users/pauldechamps/lab/dartflo/dart/viscous/Physics/XfoilFiles/' + self.xfoilFileName, 'r') as f:
                cl_xfoil=f.readline()
                cd_xfoil=f.readline()
                f.close()
            # columns : x,ue,theta,Cf,H
            data=np.loadtxt('/Users/pauldechamps/lab/dartflo/dart/viscous/Physics/XfoilFiles/'+self.xfoilFileName,skiprows=3,usecols=(1,3,4,5,6,7))

            # columns : x, delta*, theta, H, Cf, ue
            data[:,[1,2,3,4,5]]=data[:,[2,3,5,4,2]]
            data[:,]

            # Find upper/lower side separation 
            for i in range(len(data[:,0])):
                if data[i,0]<=data[i+1,0]:
                    xStag=i
                    break
        for i in range(len(wData['x'])):
            if wData['x'][i]==1:
                xTeDart=i+1
                break

        # Aerodynamic coefficients
        print('+----------------------------  Compare with XFOIL -----------------------------+')
        print('- CL; Dart: {:<.5f}, Xfoil: {:<.5f}.'.format(dartcl, float(cl_xfoil)))
        print('- CD; Dart: {:<.5f}, Xfoil: {:<.5f}.'.format(wData['Cd'], float(cd_xfoil)))

        # Plot variables
        plot_dStar=plt.figure(1)
        plt.plot(wData['x'][:xTeDart],wData['delta*'][:xTeDart],'b-',label=r'DARTflo upper')
        plt.plot(wData['x'][xTeDart:],wData['delta*'][xTeDart:],'r-',label=r'DARTflo lower')
        plt.xlim([0,1])
        if wakeblVectors is not None:
            plt.plot(xWake, wakeblVectors[0],label=r'DARTflo wake')
            plt.xlim([0,2])
        
        if xFoilIn:
            plt.plot(data[:xStag,0],data[:xStag,1],'b--',label=r'Xfoil upper')
            plt.plot(data[xStag:,0],data[xStag:,1],'r--',label=r'Xfoil lower')
            #plt.plot(data[xWake:,0],data[xWake:,1],'r--',label=r'Xfoil Wake')
        plt.legend()
        plt.xlabel('$x/c$')
        plt.ylabel('$\delta^*$')

        # H
        plot_H=plt.figure(3)
        plt.plot(wData['x'][:xTeDart],wData['H'][:xTeDart],'b-',label=r'DARTflo upper')
        plt.plot(wData['x'][xTeDart:],wData['H'][xTeDart:],'r-',label=r'DARTflo lower')
        plt.xlim([0,1])
        if wakeblVectors is not None:
            plt.plot(xWake, wakeblVectors[2],label=r'DARTflo wake')
            plt.xlim([0,2])

        if xFoilIn:
            plt.plot(data[:xStag,0],data[:xStag,3],'b--',label=r'Xfoil upper')
            plt.plot(data[xStag:,0],data[xStag:,3],'r--',label=r'Xfoil lower')
            #plt.plot(data[xWake:,0],data[xWake:,3],'r--',label=r'Xfoil Wake')
        plt.legend()
        plt.xlabel('$x/c$')
        plt.ylabel('$H$')
        # Cf
        plot_Cf=plt.figure(4)
        plt.plot(wData['x'][:xTeDart], wData['Cf'][:xTeDart],'b-',label=r'DARTflo upper')
        plt.plot(wData['x'][xTeDart:], wData['Cf'][xTeDart:],'r-',label=r'DARTflo lower')
        plt.xlim([0,1])
        if wakeblVectors is not None:
            plt.plot(xWake, wakeblVectors[6],label=r'DARTflo wake')
            plt.xlim([0,2])

        if xFoilIn:
            plt.plot(data[:xStag,0],data[:xStag,4],'b--',label=r'Xfoil upper')
            plt.plot(data[xStag:,0],data[xStag:,4],'r--',label=r'Xfoil lower')
            #plt.plot(data[xWake:,0],data[xWake:,4],'r--',label=r'Xfoil Wake')
        plt.legend()
        plt.xlabel('$x/c$')
        plt.ylabel('$C_f$')

        plt.show()

